# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)


class PochSegmentH(HeaderSegment):
    sender_id = HprimMultiple(
        5,
        instrument_name=HprimString(1),
        software_version=HprimString(2),
        user_instrument=HprimString(6),
        ps_code=HprimString(7),
    )
    version = HprimString(13, default="E1394-97")

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class PochSegmentP(Segment):
    # le segment P est vide
    orders = HprimForeign("O", "patient")


class PochSegmentO(Segment):
    instrument_specimen_id = HprimMultiple(
        4, sample_id=HprimString(3), sample_id_attributes=HprimString(4)
    )
    analysis_parameter_id = HprimList(
        5, basefield=HprimMultiple(0, parameter_name=HprimString(5))
    )
    action_code = HprimString(12, default="N")
    report_type = HprimString(26)
    results = HprimForeign("R", "order")


class PochSegmentR(Segment):
    parameter_name = HprimString((3, 5))
    dilution_ratio = HprimString((3, 6))
    value = HprimString(4)
    units = HprimString(5)
    ranges = HprimString(6)
    flag_error = HprimString(7)
    date = HprimDate(13)


class PochSegmentL(Segment):
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "poch", "label": "Poch Sysmex"}
    SEGMENT_CLASS_MAP = {
        "H": PochSegmentH,
        "P": PochSegmentP,
        "O": PochSegmentO,
        "R": PochSegmentR,
        "L": PochSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = PochSegmentH(type_segment="H", **kwargs)
        message.end = PochSegmentL(type_segment="L")
        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                tube = seg_o.instrument_specimen_id.sample_id.strip()
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        type="control", sample_id=tube, control_lot=tube
                    )
                else:
                    sample = worklist.get_or_create(type="patient", sample_id=tube)
                for seg_r in seg_o.results:
                    sample.results.add(
                        code=seg_r.parameter_name,
                        date=seg_r.date,
                        flags=seg_r.flag_error,
                        value=seg_r.value,
                    )
        return worklist
